;; 
;; This function can be used with Darrell Kindred's hot-url package 
;; to retrieve URLs using w3 (the emacs World Wide Web browser) rather 
;; than via xmosaic. It works under gnu-emacs version 18 or lucid emacs.
;; 
;; Just do
;;    (setq hot-url-fetch-url-function 'fetch-url-using-w3)
;;
;; Arup Mukherjee 11/93
;;
;; 12-Feb-95 - made compatible w/ XEmacs 19.12 (frame stuff) -dkindred
;; 25-Jun-95 - replaced "devicep" with "device-type" for released 19.12 - arup

(defvar last-w3-frame-made nil
  "don't use this")

(defvar last-www-frame-made nil
  "don't use this")

(cond 
 ((and (fboundp 'get-frame-for-buffer)  
       (or (not (fboundp 'device-type))
	   (memq 'x (mapcar 'device-type (device-list))))
       (put 'w3-mode 'frame-name 'WWW)))
 ((fboundp 'get-screen-for-buffer) (put 'w3-mode 'screen-name 'WWW)))

(defun fetch-url-using-w3 (buf b e pfx url)
  "Fetch a document with the given URL using the w3 package.
If PFX is >1, prompt the user using the given URL as the default."
  (if (or (> pfx 1)
	  (null url))
      (setq url (read-from-minibuffer "URL: " url)))
  (require 'w3)
  (message "Fetching %s with w3..." url)
  (if (not (cond ((fboundp 'frame-live-p) 
		  (frame-live-p last-www-frame-made))
		 ((fboundp 'live-screen-p) 
		  (live-screen-p last-www-frame-made))))
      (setq last-www-frame-made nil))
  (if (or (not (or (fboundp 'get-frame-for-buffer)
		   (fboundp 'get-screen-for-buffer)))
	  (and (fboundp 'device-type) 
	       (eq 'tty (device-type (selected-device)))))
      (w3-fetch url)
    (let ((owindow (selected-window))
	  (w3-mutable-windows t)
	  (obuf (if (cond ((fboundp 'framep) (framep last-www-frame-made))
			  ((fboundp 'screenp) (screenp last-www-frame-made)))
		    (window-buffer 
		     (screen-selected-window last-www-frame-made))
		  (get-buffer-create "*scratch*"))))
      (w3-fetch url)
      (setq w3-current-last-buffer obuf)
      (setq last-www-frame-made 
	    (cond ((fboundp 'selected-frame) (selected-frame))
		  ((fboundp 'selected-screen) (selected-screen))))
      ;; this is a bit gross, since it will usually be selecting
      ;; another frame, and keypresses can (I think) get dropped.
      (select-window owindow))))

(provide 'w3-fetch-url)
